---
title: Modules, Packages, and Imports
description: "Resumen del capítulo 10 del libro «Learning Go: An idiomatic approach to real-world Go programming, 2nd Edition»."
date: 2024-03-13
last_update:
id: 20240313113936
references:
  books:
  - book:
    - title: "Learning Go: An idiomatic approach to real-world Go programming"
    - author: Bodner, J.
    - edition: 2nd. Edition
    - publisher: O'Reilly Media
    - year: 2024
    - isbn: 9781098139292
    - bibliography: "Bodner, J. (2024). «Learning Go: An idiomatic approach to real-world Go programming». 2nd. Edition. O'Reilly Media."
taxonomies:
  categories:
    - development
  tags:
    - golang
license: © All rights reserved
---
Most modern programming languages have a system for organizing code into namespaces and libraries, and Go is no exception. In this chapter, you'll learn how to organize code with packages and modules, how to import them, how to work with third-party libraries, and how to create libraries of your own.

## Repositories, Modules, and Packages

*Library management in Go is based around three concepts: repositories, modules, and packages.* A **repository** is a place in a version control system where the source code for a project is stored. A **module** is a bundle of Go source code that's distributed and versioned as a single unit. Modules are stored in a repository. Modules consist of one or more **packages**, which are directories of source code. Packages give a module organization and structure.

> :memo: **Note:** While you can store more than one module in a repository, *it is discouraged*. Everything within a module is versioned together. Maintaining two modules in one repository requires you to track separate versions for two different modules in a single repository.

Unfortunately, different programming languages use these words to represent different concepts. While packages in Java and Go are similar, a Java repository is a centralized place to store multiple *artifacts* (the analogue of a Go module). Node.js and Go swap the meanings of the terms: a Node.js package is similar to what Go calls a module, and a Go package is similar to a Node.js module. The terminology certainly can be confusing at first, but as you get more comfortable with Go, the terms will seem more familiar.

**Before using code from packages outside the standard library, you need to make sure that you have a properly created module.** Every module has a globally unique identifier. This is not unique to Go. Java defines globally unique package declarations by using the reverse domain name convention (`com.companyname.projectname.library`).

In Go, this name is called a *module path*. It is usually based on the repository where the module is stored.

> :memo: **Note:** Back in "Your First Go Program", you created a module whose name was `hello_world`, which is obviously not globally unique. This is fine if you are creating a module for local use only. **If you put a module with a nonunique name into a source code repository, it cannot be imported by another module.**

## Using go.mod

**A directory tree of Go source code becomes a module when there's a valid *go.mod* file in it.** Rather than create this file manually, use the subcommands of the `go mod`
command to manage modules. The command `go mod init MODULE_PATH` creates the *go.mod* file that makes the current directory the root of a module. The `MODULE_PATH` is the globally unique name that identifies your module. The module path is case-sensitive. **To reduce confusion, do not use uppercase letters within it.**

Take a look at the contents of a *go.mod* file:

```
module github.com/learning-go-book-2e/money

go 1.21

require (
    github.com/learning-go-book-2e/formatter v0.0.0-20220918024742-18...
    github.com/shopspring/decimal v1.3.1
)

require (
    github.com/fatih/color v1.13.0 // indirect
    github.com/mattn/go-colorable v0.1.9 // indirect
    github.com/mattn/go-isatty v0.0.14 // indirect
    golang.org/x/sys v0.0.0-20210630005230-0f9fa26af87c // indirect
)
```

Every *go.mod* file starts with a `module` directive that consists of the word `module` and the module's unique path. Next, the *go.mod* file specifies the minimum compatible version of Go with the `go` directive. All source code within the module must be compatible with the specified version.

### Use the go Directive to Manage Go Build Versions

*What happens if the go directive specifies a version of Go that's newer than what's installed?* If you have Go 1.20 or earlier installed, the newer Go version is ignored and you get the features of the installed version. If you have Go 1.21 or newer installed, the default behavior is to download the newer version of Go and use it to build your code. You can control this behavior in Go 1.21 and newer with the `toolchain` directive and the `GOTOOLCHAIN` environment variable. You can assign them the following values:

- `auto`, which downloads newer versions of Go. (This is the default behavior of Go 1.21 and later.)
- `local`, which restores the behavior of Go releases before 1.21.
- A specific Go version (such as `go1.20.4`), which means that specific version will be downloaded and used to build the program.

If both the `GOTOOLCHAIN` environment variable and the `toolchain` directive are set, the value assigned to `GOTOOLCHAIN` is used.

Complete details on the `go` directive, the `toolchain` directive, and the `GOTOOLCHAIN` environment variable are found in the [official Go toolchain documentation](https://go.dev/doc/toolchain).

> :memo: **Note:** Go 1.22 introduces the first backward-breaking change to the language. When using Go 1.22 or later, if the `go` directive is set to `1.22` or higher, a `for` loop creates a new index and value variable on each iteration. This behavior is applied per module. The value of the `go` directive in each imported module determines the language level for that module.

### The require Directive

The next sections in a *go.mod* file are the `require` directives. The `require` directives are present only if your module has dependencies. They list the modules that your module depends on and the minimum version required for each one. The first `require` section lists the direct dependencies of your module. The second one lists the dependencies of the dependencies of your module. Each line in this section ends with an `// indirect`
comment. There's no functional difference between modules marked as indirect and those that aren't; it's just documentation for people when they look at the *go.mod* file.

While the `module`, `go`, and `require` directives are the ones most commonly used in a *go.mod* file, there are three others directives as well. I will cover the `replace` and `exclude` directives in [[20240317111945|Overriding Dependencies]], and I'll cover the `retract` directive in [[20240101000000|Retracting a Version of Your Module]].

[TODO: Incluir referencias a las secciones: «Overriding Dependencies» y «Retracting a Version of Your Module» y actualizar los enlaces correspondientes]: #



